from sklearn.metrics import mean_squared_error, mean_absolute_percentage_error
from xgboost import XGBRegressor
import pandas as pd
import scipy.stats as st


#Load dataset
converted_data = pd.DataFrame("combinatory_featured_data.csv", header=None)

#Set parameters
n_hours_before = 48
n_days_testing = 90
n_days_forecasting = 14
n_samples_testing = 24*n_days_testing
n_samples_forecasting = 24*n_days_forecasting
n_samples_moving = 24
n_days_to_retrain = 7

#Define model
model = XGBRegressor(objective='reg:squarederror', n_estimators=100, max_depth=5)

#Evaluation
retrain_counter = 0
metrics = []
actual_values = []
predictions = []

for d in range(n_days_testing-n_days_forecasting):
    i = d * 24
    current_data = converted_data[:-n_samples_testing+i, :]
    future_data = converted_data[-n_samples_testing+i:, :]
    
    X_train, y_train = current_data[:, :-1], current_data[:, -1]
    X_test, y_test = future_data[:n_samples_forecasting+1, :-1], future_data[:n_samples_forecasting+1, -1]

    if (retrain_counter == 0):
        print("Retraining using", len(y_train), "data points")
        model.fit(X_train, y_train)
    
    retrain_counter += 1
    if (retrain_counter == n_days_to_retrain):
        retrain_counter = 0
    
#    y_predicted = model.predict(X_test)
    X_last = current_data[-1,-n_hours_before-1:-1]
    y_last = current_data[-1,-1]
    y_actual = []
    y_forecasted = []
    for j in range(n_samples_forecasting):
        X_last = np.roll(X_last, -1)
        X_last[-1] = y_last
        X_sample_test = future_data[j, :-1]
        y_sample_test = future_data[j, -1]
        X_sample_test[-n_hours_before:] = X_last
        y_sample_hat = model.predict(np.asarray([X_sample_test]))
        
        y_last = y_sample_hat
        y_actual.append(y_sample_test)
        y_forecasted.append(y_sample_hat)

    actual_values.append(y_actual)
    predictions.append(y_forecasted)
    metrics.append(mean_squared_error(y_actual, y_forecasted, squared=False))
    metrics.append(mean_absolute_percentage_error(y_actual, y_forecasted))

performances = np.reshape(np.array(metrics),(2,-1),order="F")

print(round(performances[0].mean(), 2))
print(round(performances[0].std(), 2))
print(st.t.interval(alpha=0.95, 
                    df=len(performances[0])-1,
                    loc=np.mean(performances[0]),
                    scale=st.sem(performances[0])))

print(round(performances[1].mean(), 2))
print(round(performances[1].std(), 2))
print(st.t.interval(alpha=0.95, 
                    df=len(performances[1])-1,
                    loc=np.mean(performances[1]),
                    scale=st.sem(performances[1])))