from sklearn.metrics import mean_squared_error, mean_absolute_percentage_error
from xgboost import XGBRegressor
import matplotlib.pyplot as plt
import pandas as pd


#Load dataset
converted_data = pd.DataFrame("combinatory_featured_data.csv", header=None)

# Set parameters
n_hours_before = 48
converted_train_data = converted_data[:-90*24,:]
n_days_validating = 60
n_days_forecasting = 14
n_samples_validating = 24*n_days_validating
n_samples_forecasting = 24*n_days_forecasting
n_samples_moving = 24
n_days_to_retrain = 7

# Define models
xgbmodels = [XGBRegressor(objective='reg:squarederror', max_depth=2),
            XGBRegressor(objective='reg:squarederror', max_depth=3),
            XGBRegressor(objective='reg:squarederror', max_depth=4),
            XGBRegressor(objective='reg:squarederror', max_depth=5)]

# Tuning
validated_xgbmodels = {}
model_names = ["xgb_md2", "xgb_md3", "xgb_md4", "xgb_md5"]

for m in range(len(xgbmodels)):
    retrain_counter = 0
    metrics = []
    actual_values = []
    predictions = []

    for d in range(n_days_validating-n_days_forecasting):
        i = d * n_samples_moving
        current_data = converted_train_data[:-n_samples_validating+i, :]
        future_data = converted_train_data[-n_samples_validating+i:, :]

        X_train, y_train = current_data[:, :-1], current_data[:, -1]
        X_val, y_val = future_data[:n_samples_forecasting+1, :-1], future_data[:n_samples_forecasting+1, -1]

        if (retrain_counter == 0):
            print("Retraining using", len(y_train), "data points")
            model = xgbmodels[m]
            model.fit(X_train, y_train)

        retrain_counter += 1
        if (retrain_counter == n_days_to_retrain):
            retrain_counter = 0

        X_last = current_data[-1,-n_hours_before-1:-1]
        y_last = current_data[-1,-1]
        y_actual = []
        y_forecasted = []
        for j in range(n_samples_forecasting):
            X_last = np.roll(X_last, -1)
            X_last[-1] = y_last
            X_sample_val = future_data[j, :-1]
            y_sample_val = future_data[j, -1]
        
            X_sample_val[-n_hours_before:] = X_last
            y_sample_hat = model.predict(np.asarray([X_sample_val]))

            y_last = y_sample_hat
            y_actual.append(y_sample_val)
            y_forecasted.append(y_sample_hat)

        actual_values.append(y_actual)
        predictions.append(y_forecasted)
        metrics.append(mean_squared_error(y_actual, y_forecasted, squared=False))
        metrics.append(mean_absolute_percentage_error(y_actual, y_forecasted))
    validated_xgbmodels[model_names[m]] = metrics
    
xgb_performances = [np.reshape(np.array(validated_xgbmodels[model]),(2,-1),order="F") for model in model_names]